'----------------------------------------------------------------------------------------------------------------------
'
' Demo and test harness for heater_fft
'
' Michael Rychlik 2011-01-14
'
' This file is released under the terms of the MIT license. See below.
'
' 2010-12-16    v0.1  Initial draft.
'
' 2010-12-18    v0.2  First working version.
'
' 2010-12-18    v0.3  For heater_fft v0.3
'
' 2010-12-21    v0.4  For heater_fft v0.3
'
' 2010-12-28    v0.5  For heater_fft v0.3
'
' 2011-01-14    v1.0  For heater_fft v0.3
'                     Added copyright and MIT licence statemets.
'
'----------------------------------------------------------------------------------------------------------------------
OBJ
    'Set your platform configuration in userdef.spin
    'Ideally, your userdefs.spin can live in the library path,
    'and you can simply remove a new package's userdefs.spin
    'to enable your own definitions.
    def  : "userdefs.spin"

    fft  : "heater_fft.spin"

    'User interface port
    ser  : "FullDuplexSerialPlus"

'----------------------------------------------------------------------------------------------------------------------

'----------------------------------------------------------------------------------------------------------------------
CON
    _clkmode      = def#_clkmode
    _xinfreq      = def#_xinfreq

#define PASM_BUTTERFLIES
'#define SPIN_BUTTERFLIES

'----------------------------------------------------------------------------------------------------------------------

'----------------------------------------------------------------------------------------------------------------------
VAR
    'Pointers to bx and by FFT buffers
    long bx
    long by
'----------------------------------------------------------------------------------------------------------------------

'----------------------------------------------------------------------------------------------------------------------
PUB testFFT | startTime, endTime
'Test harness for the FFT
'
    'Start the terminal
    ser.start(def#conRxPin, def#conTxPin, def#conMode, def#conBaud)
    ser.str(string("heater_fft v1.0"))
    newline

    fft.start

    'Get pointers to internal FFT buffers
    bx := fft.getbx
    by := fft.getby

    'Input some data
    fillInput
    'printreals

    'Radix-2 Decimation In Time, the bit-reversal step.
    'This take only ~16% of the total FFT time.
    fft.decimate

    'The actual Fourier transform work.
    startTime := cnt
    fft.butterflies
    endTime := cnt

    'Print resulting spectrum
    printSpectrum

    ser.str(string("1024 point butterfly run time = "))
    ser.dec((endTime - startTime) / (clkfreq / 1000))
    ser.str(string("ms"))
    newline
    newline
    repeat

PUB printReals | i
    repeat i from 0 to fft#FFT_SIZE/2 - 1
        ser.dec(long[@bx + i * 4])
        newline

PUB printSpectrum  | i, real, imag, magnitude
'Spectrum is available in first half of the buffers after FFT.
    ser.str(string("Freq. Magnitude"))
    newline
    repeat i from 0 to (fft#FFT_SIZE / 2)
        'Scale down by half FFT size, back to original signal input range
        real := long[bx + i * 4] / (fft#FFT_SIZE / 2)
        imag := long[by + i * 4] / (fft#FFT_SIZE / 2)

        'Frequency magnitude is square root of cos part sqaured plus sin part squared
        magnitude := ^^((real * real) + (imag * imag))

        ser.dec(i)
        ser.str(string(" "))
'        ser.hex(real, 8)
'        ser.str(string(" "))
'        ser.hex(imag, 8)
'        ser.str(string(" "))
        ser.dec(magnitude)
        newline


PUB fillInput | k, r, sx, sy
'Fill buffer bx with samples of of an imput signal and clear by.
'WARNING: Keep the peek amplitude < 2048.
    repeat k from 0 to (fft#FFT_SIZE - 1)
        'Two frequencies of the waveform defined in input
        sx:= bx + k * 4
        sy:= by + k * 4
        long[sx] := (~~input[(3*k) // 16] / 4) - 1
        long[sx] += (~~input[(5*k) // 16] / 4) - 1

        'The highest frequency
        if (k & 1)
          long[sx] += 200
        else
          long[sx] += -200
        'A DC level
        long[sx] += 200

        'Clear Y array.
        long[sy] := 0


PUB newline
    ser.tx(13)
    ser.tx(10)


'----------------------------------------------------------------------------------------------------------------------
'Here starts Version 2
'----------------------------------------------------------------------------------------------------------------------

'----------------------------------------------------------------------------------------------------------------------
DAT
'For testing define 16 samples  of an input wave form here.
input long 4096, 3784, 2896, 1567, 0, -1567, -2896, -3784, -4096, -3784, -2896, -1567, 0, 1567, 2896, 3784
'----------------------------------------------------------------------------------------------------------------------

'----------------------------------------------------------------------------------------------------------------------
'    This file is distributed under the terms of the The MIT License as follows:
'
'    Copyright (c) 2011 Michael Rychlik
'
'    Permission is hereby granted, free of charge, to any person obtaining a copy
'    of this software and associated documentation files (the "Software"), to deal
'    in the Software without restriction, including without limitation the rights
'    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
'    copies of the Software, and to permit persons to whom the Software is
'    furnished to do so, subject to the following conditions:
'
'    The above copyright notice and this permission notice shall be included in
'    all copies or substantial portions of the Software.
'
'    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
'    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
'    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
'    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
'    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
'    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
'    THE SOFTWARE.
'----------------------------------------------------------------------------------------------------------------------

'----------------------------------------------------------------------------------------------------------------------
'The end.

